@@ -1,4 +1,4 @@
-#!C:\perl\bin\perl.exe
+#!/usr/bin/perl -w
 package Net::BitTorrent;
 {
     use strict;
@@ -7,19 +7,20 @@ package Net::BitTorrent;
     use List::Util qw[max];
     use Time::HiRes;
     use Socket qw[/inet_/ SOCK_STREAM SOCK_DGRAM SOL_SOCKET PF_INET SOMAXCONN
-        /pack_sockaddr_in/];
+        /pack_sockaddr_in/ SO_REUSEADDR];
     use Carp qw[carp];
+    use Digest::SHA qw[sha1_hex];
     use POSIX qw[];
     sub _EWOULDBLOCK { $^O eq q[MSWin32] ? 10035 : POSIX::EWOULDBLOCK() }
     sub _EINPROGRESS { $^O eq q[MSWin32] ? 10036 : POSIX::EINPROGRESS() }
     use lib q[../../lib];
+    use Net::BitTorrent::Util qw[:bencode :compact];
     use Net::BitTorrent::Torrent;
     use Net::BitTorrent::Peer;
     use Net::BitTorrent::DHT;
     use Net::BitTorrent::Version;
     use version qw[qv];
-    our $SVN = q[$Id: BitTorrent.pm 39 2008-11-26 15:49:02Z sanko@cpan.org $];
-    our $UNSTABLE_RELEASE = 4; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new((qw$Rev: 39 $)[1])->numify / 1000), $UNSTABLE_RELEASE);
+    our $VERSION_BASE = 49; our $UNSTABLE_RELEASE = 3; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new(($VERSION_BASE))->numify / 1000), $UNSTABLE_RELEASE);
     my (@CONTENTS)
         = \my (%_tcp,                  %_udp,
                %_schedule,             %_tid,
@@ -37,6 +38,24 @@ package Net::BitTorrent;
         my ($class, $args) = @_;
         my $self = bless \$class, $class;
         my ($host, @ports) = (q[0.0.0.0], (0));
+
+        # Defaults
+        $_max_ul_rate{refaddr $self}          = 0;
+        $_k_ul{refaddr $self}                 = 0;
+        $_max_dl_rate{refaddr $self}          = 0;
+        $_k_dl{refaddr $self}                 = 0;
+        $_peers_per_torrent{refaddr $self}    = 100;
+        $_half_open{refaddr $self}            = 8;
+        $_connections_per_host{refaddr $self} = 2;
+        $_torrents{refaddr $self}             = {};
+        $_tid{refaddr $self}                  = qq[\0] x 5;
+        $_use_dht{refaddr $self}              = 1;
+
+        # Internals
+        $_connections{refaddr $self} = {};
+        $_schedule{refaddr $self}    = {};
+        $_dht{refaddr $self} = Net::BitTorrent::DHT->new({Client => $self});
+        $_peerid{refaddr $self} = Net::BitTorrent::Version::gen_peerid();
         if (defined $args) {
             if (ref($args) ne q[HASH]) {
                 carp q[Net::BitTorrent->new({}) requires ]
@@ -53,33 +72,39 @@ package Net::BitTorrent;
                 )
                 : @ports;
         }
-        $_peers_per_torrent{refaddr $self}    = 100;
-        $_connections_per_host{refaddr $self} = 2;
-        $_half_open{refaddr $self}            = 8;
-        $_max_dl_rate{refaddr $self}          = 0;
-        $_k_dl{refaddr $self}                 = 0;
-        $_max_ul_rate{refaddr $self}          = 0;
-        $_k_ul{refaddr $self}                 = 0;
-        $_torrents{refaddr $self}             = {};
-        $_tid{refaddr $self}                  = qq[\0] x 5;
-        $_use_dht{refaddr $self}              = 1;
-        $_dht{refaddr $self} = Net::BitTorrent::DHT->new({Client => $self});
-        $_peerid{refaddr $self}      = Net::BitTorrent::Version::gen_peerid();
-        $_connections{refaddr $self} = {};
+
+        # Try opening a matching set of ports
+        for my $port (@ports) {
+            last
+                if $self->_socket_open_tcp($host, $port)
+                    && $self->_socket_open_udp($host, $port);
+        }
+
+        # Clear everything just in case
         $self->_reset_bandwidth;
-        for my $port (@ports) { last if $self->_socket_open($host, $port) }
         weaken($REGISTRY{refaddr $self} = $self);
+        $$self = $_peerid{refaddr $self};
         return $self;
     }
 
     # Accessors | Private
-    sub _tcp         { return $_tcp{refaddr +shift} }
-    sub _udp         { return $_udp{refaddr +shift} }
-    sub _connections { return $_connections{refaddr +shift} }
-    sub _max_ul_rate { return $_max_ul_rate{refaddr +shift} }
-    sub _max_dl_rate { return $_max_dl_rate{refaddr +shift} }
-    sub _dht { my ($s) = @_; return $_udp{refaddr $s} && $_dht{refaddr $s} }
-    sub _use_dht { return $_use_dht{refaddr +shift}; }
+    sub _tcp               { return $_tcp{refaddr +shift} }
+    sub _udp               { return $_udp{refaddr +shift} }
+    sub _connections       { return $_connections{refaddr +shift} }
+    sub _max_ul_rate       { return $_max_ul_rate{refaddr +shift} }
+    sub _max_dl_rate       { return $_max_dl_rate{refaddr +shift} }
+    sub _peers_per_torrent { return $_peers_per_torrent{refaddr +shift} }
+    sub _half_open         { return $_half_open{refaddr +shift} }
+
+    sub _connections_per_host {
+        return $_connections_per_host{refaddr +shift};
+    }
+    sub _dht { return $_dht{refaddr +shift} }
+
+    sub _use_dht {
+        my ($s) = @_;
+        return $_udp{refaddr $s} && $_use_dht{refaddr $s};
+    }
 
     sub _tcp_port {
         my ($self) = @_;
@@ -112,17 +137,11 @@ package Net::BitTorrent;
             = unpack_sockaddr_in(getsockname($_udp{refaddr $self}));
         return inet_ntoa($packed_ip);
     }
-    sub _peers_per_torrent { return $_peers_per_torrent{refaddr +shift} }
-    sub _half_open         { return $_half_open{refaddr +shift} }
-
-    sub _connections_per_host {
-        return $_connections_per_host{refaddr +shift};
-    }
 
     # Setters | Private
     sub _set_max_ul_rate {    # BYTES per second
         my ($self, $value) = @_;
-        if (not defined $value or $value !~ m[^\d+$]) {
+        if (not defined $value or $value !~ m[^\d+$] or !$value) {
             carp
                 q[Net::BitTorrent->_set_max_ul_rate( VALUE ) requires an integer value];
             return;
@@ -140,6 +159,36 @@ package Net::BitTorrent;
         return $_max_dl_rate{refaddr $self} = $value;
     }
 
+    sub _set_peers_per_torrent {
+        my ($self, $value) = @_;
+        if (not defined $value or $value !~ m[^\d+$] or $value < 1) {
+            carp
+                q[Net::BitTorrent->_set_peers_per_torrent( VALUE ) requires an integer value];
+            return;
+        }
+        return $_peers_per_torrent{refaddr $self} = $value;
+    }
+
+    sub _set_half_open {
+        my ($self, $value) = @_;
+        if (not defined $value or $value !~ m[^\d+$] or $value < 1) {
+            carp
+                q[Net::BitTorrent->_set_half_open( VALUE ) requires an integer value];
+            return;
+        }
+        return $_half_open{refaddr $self} = $value;
+    }
+
+    sub _set_connections_per_host {
+        my ($self, $value) = @_;
+        if (not defined $value or $value !~ m[^\d+$] or $value < 1) {
+            carp
+                q[Net::BitTorrent->_set_connections_per_host( VALUE ) requires an integer value];
+            return;
+        }
+        return $_connections_per_host{refaddr $self} = $value;
+    }
+
     sub _set_use_dht {
         my ($self, $value) = @_;
         if (not defined $value or $value !~ m[^[10]$]) {
@@ -151,8 +200,8 @@ package Net::BitTorrent;
     }
 
     # Accessors | Public
-    sub peerid   { return $_peerid{refaddr +shift} }
-    sub torrents { return $_torrents{refaddr +shift} }
+    sub peerid   { my ($self) = @_; return $_peerid{refaddr $self} }
+    sub torrents { my ($self) = @_; return $_torrents{refaddr $self} }
 
     # Methods | Public
     sub do_one_loop {
@@ -233,29 +282,30 @@ package Net::BitTorrent;
         return delete $_connections{refaddr $self}{fileno $socket};
     }
 
-    sub _socket_open {
+    sub _socket_open_tcp {
         my ($self, $host, $port) = @_;
         if (   not $self
             || not blessed $self
             || not $self->isa(q[Net::BitTorrent]))
         {   carp
-                q[Net::BitTorrent->_socket_open(HOST, PORT) requires a blessed object];
+                q[Net::BitTorrent->_socket_open_tcp(HOST, PORT) requires a blessed object];
             return;
         }
         if ((!$_tcp{refaddr $self}) && (!$host)) {
-            carp q[Net::BitTorrent::__socket_open( ) ]
+            carp q[Net::BitTorrent::_socket_open_tcp( ) ]
                 . q[requires a hostname];
             return;
         }
-        if (   (!$_tcp{refaddr $self}) && (!defined $port)
-            || (defined $port and $port !~ m[^\d+$]))
-        {   carp q[Net::BitTorrent::__socket_open( ) ]
+        if (defined $port and $port !~ m[^\d+$]) {
+            carp q[Net::BitTorrent::_socket_open_tcp( ) ]
                 . q[requires an integer port number];
             return;
         }
         my $_packed_host = undef;
         $host ||= q[0.0.0.0];
         $port ||= 0;
+        $port =~ m[^(\d+)$];
+        $port = $1;
         if (    $host
             and $host
             !~ m[^(?:(?:(?:25[0-5]|2[0-4][0-9]|[0-1]?[0-9]{1,2})[.]?){4})$])
@@ -265,45 +315,100 @@ package Net::BitTorrent;
             $_packed_host = $addrs[0];
         }
         else { $_packed_host = inet_aton($host) }
-        if (!$_tcp{refaddr $self}) {
-            socket(my ($_tcp), PF_INET, SOCK_STREAM, getprotobyname(q[tcp]))
-                or return;
-            bind($_tcp, pack_sockaddr_in($port, $_packed_host))
-                or return;
-            listen($_tcp, 1) or return;
-            $_connections{refaddr $self}{fileno $_tcp} = {Object => $self,
-                                                          Mode   => q[ro]
-                }
-                or return;
-            $_tcp{refaddr $self} = $_tcp;
+        socket(my ($_tcp), PF_INET, SOCK_STREAM, getprotobyname(q[tcp]))
+            or return;
+
+       # - What is the difference between SO_REUSEADDR and SO_REUSEPORT?
+       #    [http://www.unixguide.net/network/socketfaq/4.11.shtml]
+       # - setsockopt - what are the options for ActivePerl under Windows NT?
+       #    [http://perlmonks.org/?node_id=63280]
+       #      setsockopt($_tcp, SOL_SOCKET, SO_REUSEADDR, pack(q[l], 1))
+       #         or return;
+       # SO_REUSEPORT is undefined on Win32... Boo...
+       #if ($reuse_port and defined SO_REUSEPORT) {       # XXX - undocumented
+       #   setsockopt($_udp, SOL_SOCKET, SO_REUSEPORT, pack(q[l], 1))
+       #       or return;
+       #}
+        bind($_tcp, pack_sockaddr_in($port, $_packed_host))
+            or return;
+        listen($_tcp, 1) or return;
+        $_connections{refaddr $self}{fileno $_tcp} = {Object => $self,
+                                                      Mode   => q[ro],
+            }
+            or return;
+        if (   defined $_tcp{refaddr $self}
+            && fileno $_tcp{refaddr $self}
+            && defined $_connections{refaddr $self}
+            {fileno $_tcp{refaddr $self}})
+        {   delete $_connections{refaddr $self}{fileno $_tcp{refaddr $self}};
+            close $_tcp{refaddr $self};
         }
-        if ($_tcp{refaddr $self}
-            and !$_udp{refaddr $self})
-        {   ($port, undef)
-                = unpack_sockaddr_in(getsockname($_tcp{refaddr $self}));
-            socket(my ($_udp), PF_INET, SOCK_DGRAM, getprotobyname(q[udp]))
-                or return;
-            bind($_udp, pack_sockaddr_in($port, $_packed_host))
-                or return;
-            $_connections{refaddr $self}{fileno $_udp} = {Object => $self,
-                                                          Mode   => q[ro]
-                }
-                or return;
-            $_udp{refaddr $self} = $_udp;
+        return $_tcp{refaddr $self} = $_tcp;
+    }
+
+    sub _socket_open_udp {
+        my ($self, $host, $port) = @_;
+        if (   not $self
+            || not blessed $self
+            || not $self->isa(q[Net::BitTorrent]))
+        {   carp
+                q[Net::BitTorrent->_socket_open_udp(HOST, PORT) requires a blessed object];
+            return;
         }
-        if (    defined $_tcp{refaddr $self}
-            and defined $_udp{refaddr $self})
-        {   my ($tcp_port, $tcp_packed_ip)
-                = unpack_sockaddr_in(getsockname($_tcp{refaddr $self}));
-            my $tcp_address
-                = sprintf(q[%s:%d], inet_ntoa($tcp_packed_ip), $tcp_port);
-            my ($udp_port, $udp_packed_ip)
-                = unpack_sockaddr_in(getsockname($_udp{refaddr $self}));
-            my $udp_address
-                = sprintf(q[%s:%d], inet_ntoa($udp_packed_ip), $udp_port);
-            $$self = $tcp_address . q[ | ] . $udp_address;
+        if ((!$_tcp{refaddr $self}) && (!$host)) {
+            carp q[Net::BitTorrent::_socket_open_udp( ) ]
+                . q[requires a hostname];
+            return;
         }
-        return ($_tcp{refaddr $self} && $_udp{refaddr $self});
+        if (defined $port and $port !~ m[^\d+$]) {
+            carp q[Net::BitTorrent::_socket_open_udp( ) ]
+                . q[requires an integer port number];
+            return;
+        }
+        my $_packed_host = undef;
+        $host ||= q[0.0.0.0];
+
+        #$port = $port ? $port : $_udp{refaddr $self} ? $self->_udp_port : 0;
+        $port ||= 0;
+        $port =~ m[^(\d+)$];
+        $port = $1;
+        if (    $host
+            and $host
+            !~ m[^(?:(?:(?:25[0-5]|2[0-4][0-9]|[0-1]?[0-9]{1,2})[.]?){4})$])
+        {   my ($name, $aliases, $addrtype, $length, @addrs)
+                = gethostbyname($host)
+                or return;
+            $_packed_host = $addrs[0];
+        }
+        else { $_packed_host = inet_aton($host) }
+        socket(my ($_udp), PF_INET, SOCK_DGRAM, getprotobyname(q[udp]))
+            or return;
+
+       # - What is the difference between SO_REUSEADDR and SO_REUSEPORT?
+       #    [http://www.unixguide.net/network/socketfaq/4.11.shtml]
+       # - setsockopt - what are the options for ActivePerl under Windows NT?
+       #    [http://perlmonks.org/?node_id=63280]
+       #     setsockopt($_udp, SOL_SOCKET, SO_REUSEADDR, pack(q[l], 1))
+       #        or return;
+       # SO_REUSEPORT is undefined on Win32... Boo...
+       #if ($reuse_port and defined SO_REUSEPORT) {       # XXX - undocumented
+       #   setsockopt($_udp, SOL_SOCKET, SO_REUSEPORT, pack(q[l], 1))
+       #       or return;
+       #}
+        bind($_udp, pack_sockaddr_in($port, $_packed_host))
+            or return;
+        $_connections{refaddr $self}{fileno $_udp} = {Object => $self,
+                                                      Mode   => q[ro],
+            }
+            or return;
+        if (   $_udp{refaddr $self}
+            && fileno $_udp{refaddr $self}
+            && defined $_connections{refaddr $self}
+            {fileno $_udp{refaddr $self}})
+        {   delete $_connections{refaddr $self}{fileno $_udp{refaddr $self}};
+            close $_udp{refaddr $self};
+        }
+        return $_udp{refaddr $self} = $_udp;
     }
 
     sub _process_connections {
@@ -362,7 +467,7 @@ package Net::BitTorrent;
                                         q[Net::BitTorrent::Torrent::Tracker::UDP]
                                         )
                                         and $_->_packed_host eq $paddr
-                                } @{$_tier->_urls};
+                                } @{$_tier->urls};
                                 if (   $tracker
                                     && $tracker->_on_data($paddr, $data))
                                 {   $__UDP_OBJECT_CACHE{refaddr $self}{$paddr}
@@ -456,13 +561,11 @@ package Net::BitTorrent;
                 q[Net::BitTorrent->remove_torrent(TORRENT) requires a blessed Net::BitTorrent::Torrent object];
             return;
         }
-        for my $_peer ($torrent->_peers) {
+        for my $_peer ($torrent->peers) {
             $_peer->_disconnect(
                               q[Removing .torrent torrent from local client]);
         }
-        for my $_tracker (@{$torrent->trackers}) {
-            $_tracker->_urls->[0]->_announce(q[stopped]);
-        }
+        $torrent->stop;    # XXX - Should this be here?
         return delete $_torrents{refaddr $self}{$torrent->infohash};
     }
 
@@ -590,24 +693,33 @@ package Net::BitTorrent;
         return join q[], map {chr} @reserved;
     }
 
-    sub _as_string {
+    sub as_string {
         my ($self, $advanced) = @_;
-        my $dump = !$advanced ? $_peerid{refaddr $self} : sprintf <<'END',
+        my $dump
+            = !$advanced ? $_peerid{refaddr $self} : sprintf <<'END',
 Net::BitTorrent
 
 Peer ID: %s
-DHT is %sabled
+DHT is %sabled (Node ID: %s)
 TCP Address: %s:%d
 UDP Address: %s:%d
 ----------
 Torrents in queue: %d
 %s
+----------
 END
             $_peerid{refaddr $self},
-            ($_use_dht{refaddr $self} ? q[En] : q[Dis]),
+            $_use_dht{refaddr $self} ? q[En] : q[Dis],
+            unpack(q[H*], $_dht{refaddr $self}->node_id),
             $self->_tcp_host, $self->_tcp_port, $self->_udp_host,
-            $self->_udp_port, (scalar keys %{$_torrents{refaddr $self}}),
-            join(qq[\r\n], keys %{$_torrents{refaddr $self}});
+            $self->_udp_port,
+            (scalar keys %{$_torrents{refaddr $self}}), join(
+            qq[\r\n],
+            map {
+                sprintf q[%40s (%d: %s)], $_->infohash, $_->status,
+                    $_->_status_as_string()
+                } values %{$_torrents{refaddr $self}}
+            );
         return defined wantarray ? $dump : print STDERR qq[$dump\n];
     }
 
@@ -627,6 +739,11 @@ END
     }
     DESTROY {
         my ($self) = @_;
+        close($_tcp{refaddr $self}) if $_tcp{refaddr $self};
+        close($_udp{refaddr $self}) if $_udp{refaddr $self};
+        foreach my $conn (values %{$_connections{refaddr $self}}) {
+            close($conn->{q[Object]}->_socket) if $conn->{q[Object]};
+        }
         for (@CONTENTS) { delete $_->{refaddr $self}; }
         return delete $REGISTRY{refaddr $self};
     }
@@ -639,6 +756,19 @@ END
 
 Net::BitTorrent - BitTorrent peer-to-peer protocol class
 
+=head1 NOTICE
+
+=for html <span style="color: #F00; font-size: 1.5em;">
+
+THIS PROJECT IS ACTIVELY SEEKING DEVELOPERS. Ahem, I hate to shout but
+N::B could really use a second or third pair of eyes over the next few
+versions.  So, if you're interested and have cryptographic experience,
+or you're just familiar with Perl and want to chip in, see the notes
+on L<Joining the Project|Net::BitTorrent::Notes/"Joining the Project">
+and L<let me know|/"Author">.
+
+=for html </span>
+
 =head1 Synopsis
 
   use Net::BitTorrent;
@@ -707,23 +837,6 @@ C<undef> or an empty list.
 
 =over 4
 
-=item C<peerid ( )>
-
-Returns the L<Peer ID|Net::BitTorrent::Version/"gen_peerid ( )">
-generated to identify this L<Net::BitTorrent|Net::BitTorrent> object
-internally, with remote L<peers|Net::BitTorrent::Peer>, and
-L<trackers|Net::BitTorrent::Torrent::Tracker>.
-
-See also: wiki.theory.org (http://tinyurl.com/4a9cuv),
-L<Peer ID Specification|Net::BitTorrent::Version/"Peer ID Specification">
-
-=item C<torrents ( )>
-
-Returns the list of queued L<torrents|Net::BitTorrent::Torrent>.
-
-See also: L<add_torrent ( )|/"add_torrent ( { ... } )">,
-L<remove_torrent ( )|/"remove_torrent ( TORRENT )">
-
 =item C<add_torrent ( { ... } )>
 
 Loads a .torrent file and adds the
@@ -743,23 +856,6 @@ See also: L<torrents ( )|/"torrents ( )">,
 L<remove_torrent ( )|/"remove_torrent ( TORRENT )">,
 L<Net::BitTorrent::Torrent|Net::BitTorrent::Torrent>
 
-=item C<remove_torrent ( TORRENT )>
-
-Removes a L<Net::BitTorrent::Torrent|Net::BitTorrent::Torrent> object
-from the client's queue.
-
-=begin future
-
-Before the torrent torrent is closed, we announce to the tracker that we
-have 'stopped' downloading and a callback to store the current state is
-called.
-
-=end future
-
-See also: L<torrents ( )|/"torrents ( )">,
-L<add_torrent ( )|/"add_torrent ( { ... } )">,
-L<Net::BitTorrent::Torrent|Net::BitTorrent::Torrent>
-
 =item C<do_one_loop ( [TIMEOUT] )>
 
 Processes the internal schedule and handles activity of the various
@@ -782,6 +878,40 @@ from a file, use C<$client-E<gt>on_event( 'file_read', \&on_read )>.
 See the L<Events|/Events> section for a list of events sorted by their
 related classes.
 
+=item C<peerid ( )>
+
+Returns the L<Peer ID|Net::BitTorrent::Version/"gen_peerid ( )">
+generated to identify this L<Net::BitTorrent|Net::BitTorrent> object
+internally, with remote L<peers|Net::BitTorrent::Peer>, and
+L<trackers|Net::BitTorrent::Torrent::Tracker>.
+
+See also: wiki.theory.org (http://tinyurl.com/4a9cuv),
+L<Peer ID Specification|Net::BitTorrent::Version/"Peer ID Specification">
+
+=item C<remove_torrent ( TORRENT )>
+
+Removes a L<Net::BitTorrent::Torrent|Net::BitTorrent::Torrent> object
+from the client's queue.
+
+=begin future
+
+Before the torrent torrent is closed, we announce to the tracker that we
+have 'stopped' downloading and a callback to store the current state is
+called.
+
+=end future
+
+See also: L<torrents ( )|/"torrents ( )">,
+L<add_torrent ( )|/"add_torrent ( { ... } )">,
+L<Net::BitTorrent::Torrent|Net::BitTorrent::Torrent>
+
+=item C<torrents ( )>
+
+Returns the list of queued L<torrents|Net::BitTorrent::Torrent>.
+
+See also: L<add_torrent ( )|/"add_torrent ( { ... } )">,
+L<remove_torrent ( )|/"remove_torrent ( TORRENT )">
+
 =back
 
 =head1 Events
@@ -811,7 +941,7 @@ the following key:
 
 =item C<Address>
 
-IPv4:port address of the potential peer.
+IPv4:port (or, on rare occasions, hostname:port) address of the potential peer.
 
 =back
 
@@ -1175,7 +1305,8 @@ following keys:
 
 =item C<Torrent>
 
-The L<Net::BitTorrent::Torrent> object related to this event.
+The L<Net::BitTorrent::Torrent|Net::BitTorrent::Torrent> object related to
+this event.
 
 =item C<Index>
 
@@ -1202,6 +1333,12 @@ metadata.
 
 =back
 
+=item C<as_string ( [ VERBOSE ] )>
+
+Returns a 'ready to print' dump of the  object's data structure.  If
+called in void context, the structure is printed to C<STDERR>.
+C<VERBOSE> is a boolean value.
+
 =back
 
 =head1 Bugs
@@ -1214,33 +1351,10 @@ L<Net::BitTorrent::Notes|Net::BitTorrent::Notes> if you've found one.
 
 =head1 Notes
 
-=head2 Support and Availability
+=head2 Support Links
 
-Visit the following for support and information related to
-L<Net::BitTorrent|Net::BitTorrent>:
-
-=over 4
-
-=item The project's website
-
-For SVN info, please visit the project's home:
-http://sankorobinson.com/net-bittorrent/.
-
-=item Bug and Issue Tracker
-
-Use http://code.google.com/p/net-bittorrent/issues/list for bug
-tracking.
-
-Before sending a report, make sure the bug is reproducible.  If the
-problem requires a specific .torrent file, the issue tracker allows
-attachments.  Make sure you are using the a recent release of
-L<Net::BitTorrent|Net::BitTorrent>.  This may mean checking out the latest
-SVN commit.
-
-=back
-
-See L<Net::BitTorrent::Notes|Net::BitTorrent::Notes/"See Also">
-for links to a mailing list, SVN information, and more.
+Please refer to
+L<Net::BitTorrent::Notes|Net::BitTorrent::Notes/"Support and Information Links for C<Net::BitTorrent>">.
 
 =head2 Dependencies
 
@@ -1249,31 +1363,6 @@ L<Digest::SHA|Digest::SHA> to function and relies upon
 L<Module::Build|Module::Build> for installation.  As of perl 5.10, these
 are all CORE modules; they come bundled with the distribution.
 
-=head2 Development Policy
-
-=over 4
-
-=item * B<All APIs are subject to change.>
-
-Changes to documented or well established parts will be clearly listed
-and archived in the F<CHANGES> file.
-
-Functions and parameters that are all_lower_case_and_contain_underscores
-are typically experimental and have a very good chance of being
-depreciated in a future version.
-
-=item * B<All undocumented functionality is subject to change without notice.>
-
-L<Net::BitTorrent|Net::BitTorrent> is just asploding with incomplete bits
-of stuff so I reserve the right to change or eliminate code at any time
-without warning I<unless> functionality is defined in POD documentation.
-
-If you sift through the source and find something nifty that isn't
-described I<in full> in POD, don't expect your client to work with future
-releases.
-
-=back
-
 =head2 Examples
 
 For a demonstration of L<Net::BitTorrent|Net::BitTorrent>, see
@@ -1302,7 +1391,7 @@ decide what to do with it.
 
 L Rotger
 
-#bittorrent on Freenode for letting me idle.
+C<#bittorrent> on Freenode for letting me idle.
 
 Michel Valdrighi for b2
 
@@ -1330,6 +1419,6 @@ clarification, see http://creativecommons.org/licenses/by-sa/3.0/us/.
 Neither this module nor the L<Author|/Author> is affiliated with
 BitTorrent, Inc.
 
-=for svn $Id: BitTorrent.pm 39 2008-11-26 15:49:02Z sanko@cpan.org $
+=for svn $Id: BitTorrent.pm 56a7b7c 2009-01-27 02:13:14Z sanko@cpan.org $
 
 =cut
