@@ -1,4 +1,4 @@
-#!C:\perl\bin\perl.exe
+#!/usr/bin/perl -w
 package Net::BitTorrent::Peer;
 {
     use strict;
@@ -9,8 +9,7 @@ package Net::BitTorrent::Peer;
     use Socket qw[/F_INET/ SOMAXCONN SOCK_STREAM /inet_/ /pack_sockaddr_in/];
     use Fcntl qw[F_SETFL O_NONBLOCK];
     use version qw[qv];
-    our $SVN = q[$Id: Peer.pm 39 2008-11-26 15:49:02Z sanko@cpan.org $];
-    our $UNSTABLE_RELEASE = 0; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new((qw$Rev: 39 $)[1])->numify / 1000), $UNSTABLE_RELEASE);
+    our $VERSION_BASE = 49; our $UNSTABLE_RELEASE = 4; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new(($VERSION_BASE))->numify / 1000), $UNSTABLE_RELEASE);
     use lib q[../../../lib];
     use Net::BitTorrent::Protocol qw[:build parse_packet :types];
     use Net::BitTorrent::Util qw[:bencode];
@@ -21,7 +20,7 @@ package Net::BitTorrent::Peer;
                %_bitfield,         %_am_choking,      %_am_interested,
                %_peer_choking,     %_peer_interested, %_incoming,
                %requests_out,      %requests_in,      %_last_contact,
-               %_incoming_fastset, %_reserved_bytes
+               %_incoming_fastset, %_reserved_bytes,  %_source
         );
     my %REGISTRY;
 
@@ -84,6 +83,7 @@ END
             $_data_out{refaddr $self} = q[];
             $_data_in{refaddr $self}  = q[];
             $_incoming{refaddr $self} = 1;
+            $_source{refaddr $self}   = q[Incoming];
         }
         else {
             if ($args->{q[Address]}
@@ -100,6 +100,11 @@ END
                     q[Net::BitTorrent::Peer->new({}) requires a blessed 'Torrent'];
                 return;
             }
+            if (!$args->{q[Source]}) {
+                carp
+                    q[Net::BitTorrent::Peer->new({}) would like to know where this peer info is from];
+                return;
+            }
             my $half_open = grep {
                 $_->{q[Object]}->isa(q[Net::BitTorrent::Peer])
                     && !$_->{q[Object]}->_torrent
@@ -109,7 +114,7 @@ END
                 #warn sprintf q[%d half open sockets!], $half_open;
                 return;
             }
-            if (scalar($args->{q[Torrent]}->_peers)
+            if (scalar($args->{q[Torrent]}->peers)
                 >= $args->{q[Torrent]}->_client->_peers_per_torrent)
             {   return;
             }
@@ -130,7 +135,7 @@ END
                 or return;
             $self = bless \$args->{q[Address]}, $class;
             $_socket{refaddr $self} = $socket;
-            if (not($^O eq q[MSWin32]
+            if (not($^O eq q[MSWin32]    # set non blocking
                     ? ioctl($_socket{refaddr $self}, 0x8004667e,
                             pack(q[I], 1))
                     : fcntl($_socket{refaddr $self}, F_SETFL, O_NONBLOCK)
@@ -165,6 +170,7 @@ END
             $_data_in{refaddr $self} = q[];
             $_client{refaddr $self}->_add_connection($self, q[rw]) or return;
             $_incoming{refaddr $self} = 0;
+            $_source{refaddr $self}   = $args->{q[Source]};
         }
         if ($self) {
             ${$_am_choking{refaddr $self}}      = 1;
@@ -191,7 +197,20 @@ END
                                         Object => $self
                                        }
             );
-
+            $_client{refaddr $self}->_schedule(
+                {   Time => time + 30,
+                    Code => sub {
+                        my $s = shift;
+                        if (!$peerid{refaddr $s}) {
+                            weaken $s;
+                            $s->_disconnect(
+                                  q[Failed to complete handshake within 30s]);
+                        }
+                        return 1;
+                    },
+                    Object => $self
+                }
+            );
             if ($threads::shared::threads_shared) {
                 threads::shared::share($_bitfield{refaddr $self})
                     if defined $_bitfield{refaddr $self};
@@ -240,17 +259,22 @@ END
     sub _peer_interested { return ${$_peer_interested{refaddr +shift}} }
     sub _am_interested   { return ${$_am_interested{refaddr +shift}} }
     sub _incoming        { return $_incoming{refaddr +shift} }
+    sub _source          { return $_source{refaddr +shift} }
 
     # Methods | Private
     sub _rw {
         my ($self, $read, $write, $error) = @_;
+
+#Carp::cluck sprintf q[%s->_rw(%d, %d, %d)], __PACKAGE__, $read, $write, $error;
         if (defined $_torrent{refaddr $self}
             and !($_torrent{refaddr $self}->status & 1))
         {   weaken $self;
             $self->_disconnect(q[We aren't serving this torrent]);
             return;
         }
-        if ($error) {    #weaken $self; #$self->_disconnect($^E); return
+        if ($error) {
+            weaken $self;
+            $self->_disconnect($^E);
             return;
         }
         if (defined $_torrent{refaddr $self}
@@ -284,13 +308,13 @@ END
                                    $read,
                                    length($_data_in{refaddr $self})
             );
-            if (not $actual_read) {
+            if (!$actual_read) {
                 weaken $self;
                 $self->_disconnect(sprintf q[[%d] %s], $^E, $^E);
                 return;
             }
             else {
-                if (not defined $peerid{refaddr $self}) {
+                if (!$peerid{refaddr $self}) {
                     $_client{refaddr $self}
                         ->_event(q[peer_connect], {Peer => $self});
                 }
@@ -344,6 +368,7 @@ END
     sub __handle_handshake {
         my ($self, $payload) = @_;
         return if !defined $_socket{refaddr $self};
+        return if !defined $payload;
         $_last_contact{refaddr $self} = time;
         ($_reserved_bytes{refaddr $self}, undef, $peerid{refaddr $self})
             = @{$payload};
@@ -388,7 +413,7 @@ END
                 );
                 return;
             }
-            if (scalar($_torrent{refaddr $self}->_peers)
+            if (scalar($_torrent{refaddr $self}->peers)
                 >= $_client{refaddr $self}->_peers_per_torrent)
             {   $self->_disconnect(q[Enough peers already!]);
                 return;
@@ -551,6 +576,7 @@ END
         my ($self, $payload) = @_;
         return if !defined $_torrent{refaddr $self};
         return if !defined $_socket{refaddr $self};
+        return if !defined $payload;
         if (defined $_torrent{refaddr $self}
             and !($_torrent{refaddr $self}->status & 1))
         {   weaken $self;
@@ -558,15 +584,11 @@ END
             return;
         }
         ${$_bitfield{refaddr $self}} = $payload;
-        $_client{refaddr $self}->_event(
-            q[incoming_packet],
-            {Peer    => $self,
-             Payload => {
-
-                 # Bitfield=> $paylod # XXX - too heavy?
-             },
-             Type => BITFIELD
-            }
+        $_client{refaddr $self}->_event(q[incoming_packet],
+                                        {Peer    => $self,
+                                         Payload => {Bitfield => $payload},
+                                         Type    => BITFIELD
+                                        }
         );
         if ((unpack(q[b*], ${$_bitfield{refaddr $self}}) !~ m[1])
             && $_torrent{refaddr $self}->is_complete)
@@ -581,6 +603,7 @@ END
         my ($self, $payload) = @_;
         return if !defined $_torrent{refaddr $self};
         return if !defined $_socket{refaddr $self};
+        return if !defined $payload;
         if (defined $_torrent{refaddr $self}
             and !($_torrent{refaddr $self}->status & 1))
         {   weaken $self;
@@ -612,6 +635,7 @@ END
 
     sub __handle_piece {
         my ($self, $payload) = @_;
+        return if !defined $payload;
         if (defined $_torrent{refaddr $self}
             and !($_torrent{refaddr $self}->status & 1))
         {   weaken $self;
@@ -631,7 +655,10 @@ END
 
         if (not defined $request) {
             weaken $self;
-            $self->_disconnect(q[Handed a piece we never asked for.]);
+            $self->_disconnect(
+                sprintf
+                    q[Handed a piece we never asked for (I: %d | O: %d | L: %d).],
+                $index, $offset, $length);
             return;
         }
         $_torrent{refaddr $self}->_add_downloaded($request->{q[Length]});
@@ -650,18 +677,20 @@ END
                                         }
         );
         my $piece = $_torrent{refaddr $self}->_piece_by_index($index);
+        my $okay  = 0;
+        for my $_retry (1 .. 3) {
 
-        #use Data::Dump qw[pp];
-        #warn pp $piece;
-        if (not $_torrent{refaddr $self}->_write_data($index, $offset, \$data)
-            )
-        {   $_client{refaddr $self}->_del_socket($_socket{refaddr $self});
-            return;
+            if ($_torrent{refaddr $self}->_write_data($index, $offset, \$data)
+                )
+            {   $okay++;
+                last;
+            }
         }
-        $piece->{q[Blocks_Recieved]}->[$request->{q[_vec_offset]}] = 1;
+        return if !$okay;
+        $piece->{q[Blocks_Received]}->[$request->{q[_vec_offset]}] = 1;
         $piece->{q[Slow]}                                          = 0;
         $piece->{q[Touch]}                                         = time;
-        for my $peer ($_torrent{refaddr $self}->_peers) {
+        for my $peer ($_torrent{refaddr $self}->peers) {
             for my $x (reverse 0 .. $#{$requests_out{refaddr $peer}}) {
                 if (    (defined $requests_out{refaddr $peer}->[$x])
                     and
@@ -692,10 +721,10 @@ END
                 }
             }
         }
-        if (not grep { $_ == 0 } @{$piece->{q[Blocks_Recieved]}}) {
+        if (not grep { !$_ } @{$piece->{q[Blocks_Received]}}) {
             if ($_torrent{refaddr $self}->_check_piece_by_index($index)
                 and defined $_torrent{refaddr $self})
-            {   for my $p ($_torrent{refaddr $self}->_peers) {
+            {   for my $p ($_torrent{refaddr $self}->peers) {
                     $_data_out{$p} .= build_have($index);
                     $_client{refaddr $self}->_add_connection($p, q[rw]);
                 }
@@ -709,6 +738,7 @@ END
         my ($self, $payload) = @_;
         return if !defined $_torrent{refaddr $self};
         return if !defined $_socket{refaddr $self};
+        return if !defined $payload;
         if (defined $_torrent{refaddr $self}
             and !($_torrent{refaddr $self}->status & 1))
         {   weaken $self;
@@ -793,6 +823,7 @@ END
         my ($self, $payload) = @_;
         return if !defined $_torrent{refaddr $self};
         return if !defined $_socket{refaddr $self};
+        return if !defined $payload;
         $_client{refaddr $self}->_event(q[incoming_packet],
                                         {Payload => {Index => $payload},
                                          Peer    => $self,
@@ -848,6 +879,7 @@ END
         my ($self, $payload) = @_;
         return if !defined $_torrent{refaddr $self};
         return if !defined $_socket{refaddr $self};
+        return if !defined $payload;
         if (defined $_torrent{refaddr $self}
             and !($_torrent{refaddr $self}->status & 1))
         {   weaken $self;
@@ -864,10 +896,11 @@ END
                 {   my (undef, $packed_ip)
                         = unpack_sockaddr_in(
                                         getpeername($_socket{refaddr $self}));
-                    my $node = sprintf(q[%s:%d],
-                                       inet_ntoa($packed_ip),
-                                       $packet->{q[p]});
-                    $_client{refaddr $self}->_dht->_add_node($node);
+                    $_client{refaddr $self}->_dht->add_node(
+                                              {ip   => inet_ntoa($packed_ip),
+                                               port => $packet->{q[p]}
+                                              }
+                    );
                 }
             }
             $packet->{q[ID]} = $id;
@@ -955,14 +988,14 @@ END
         }
         for my $i (reverse(0 .. $#{$requests_out{refaddr $self}})) {
             my $request = $requests_out{refaddr $self}->[$i];
-            if (time <= ($request->{q[Timestamp]} + 30)) {
+            if (time <= ($request->{q[Timestamp]} + 60)) {
                 next;
             }
             my $piece = $_torrent{refaddr $self}
                 ->_piece_by_index($request->{q[Index]});
             delete $piece->{q[Blocks_Requested]}->[$request->{q[_vec_offset]}]
                 ->{refaddr $self};
-            if ($piece->{q[Touch]} <= time - 60) {
+            if (!$piece->{q[Touch]} || $piece->{q[Touch]} <= time - 180) {
                 $piece->{q[Slow]} = 1;
             }
             $_data_out{refaddr $self} .=
@@ -1002,11 +1035,11 @@ END
             = defined $_range
             ? [1 .. $_range]
             : [max(1, scalar(@{$requests_out{refaddr $self}})) .. max(
-                                25,
-                                int((2**21)
-                                    / $_torrent{refaddr $self}
-                                        ->raw_data->{q[info]}{q[piece length]}
-                                )
+                                   25,
+                                   int((2**21)
+                                       / $_torrent{refaddr $self}->raw_data(1)
+                                           ->{q[info]}{q[piece length]}
+                                   )
                )
             ];
     REQUEST:
@@ -1025,11 +1058,11 @@ END
                         $tmp_index++;
                         (        ($_ < 5)
                              and
-                             ($piece->{q[Blocks_Recieved]}->[$tmp_index] == 0)
+                             ($piece->{q[Blocks_Received]}->[$tmp_index] == 0)
                             )
                             ? ($tmp_index => $_)
                             : ()
-                    } @{$piece->{q[Blocks_Recieved]}};
+                    } @{$piece->{q[Blocks_Received]}};
                 INDEX:
                     for my $index (sort { $temp{$a} <=> $temp{$b} }
                                    sort { $a <=> $b } keys %temp)
@@ -1038,7 +1071,7 @@ END
                                 (defined $piece->{q[Blocks_Requested]}
                                      ->[$index]->{refaddr $self})
                                     and (
-                                    ($piece->{q[Blocks_Recieved]}->[$index])
+                                    ($piece->{q[Blocks_Received]}->[$index])
                                     or (($piece->{q[Index]} == $_->{q[Index]})
                                         and ($index == $_->{q[_vec_offset]}))
                                     )
@@ -1249,6 +1282,7 @@ END
             next
                 unless $_torrent{refaddr $self}
                     ->_check_piece_by_index($request->{q[Index]});
+            next unless $request->{q[Length]};
             $_torrent{refaddr $self}->_add_uploaded($request->{q[Length]});
             $_client{refaddr $self}->_event(
                                          q[outgoing_packet],
@@ -1315,7 +1349,7 @@ END
         return if $_torrent{refaddr $self}->status & 32;
         return if ${$_am_choking{refaddr $self}} == 0;
         if (scalar(
-                grep { $_->_am_choking == 0 } $_torrent{refaddr $self}->_peers
+                 grep { $_->_am_choking == 0 } $_torrent{refaddr $self}->peers
             ) <= 8
             )
         {   ${$_am_choking{refaddr $self}} = 0;
@@ -1357,35 +1391,46 @@ END
         return delete $_socket{refaddr $self};
     }
 
-    sub _as_string {
+    sub as_string {
         my ($self, $advanced) = @_;
         my $dump = sprintf(
             (!$advanced ? q[%s:%s (%s)] : <<'ADVANCED'),
 Net::BitTorrent::Peer
 
-Address: %s:%s
-Peer ID: %s
-Torrent: %s
+Address:     %s:%s
+Peer ID:     %s
+Torrent:     %s
+Direction:   %s
 
-Current statuss:
- Interested: %s
- Interesting: %s
- Choked: %s
- Choking: %s
+Interested:  %s
+Interesting: %s
+Choked:      %s
+Choking:     %s
 
+Progress:
+[%s]
 ADVANCED
             ($self->_host || q[]),
             ($self->_port || q[]),
             ($peerid{refaddr $self} ? $peerid{refaddr $self} : q[Unknown]),
-            (defined $_torrent{refaddr $self}
+            (  $_torrent{refaddr $self}
              ? $_torrent{refaddr $self}->infohash
              : q[Unknown]
             ),
-            (map { $_ ? q[Yes] : q[No] } ($_peer_interested{refaddr $self},
-                                          $_am_interested{refaddr $self},
-                                          $_am_choking{refaddr $self},
-                                          $_peer_choking{refaddr $self}
+            ($_incoming{refaddr $self} ? q[Incoming] : q[Outgoing]),
+            (map { $_ ? q[Yes] : q[No] } (${$_peer_interested{refaddr $self}},
+                                          ${$_am_interested{refaddr $self}},
+                                          ${$_am_choking{refaddr $self}},
+                                          ${$_peer_choking{refaddr $self}}
              )
+            ),
+            ($_torrent{refaddr $self}
+             ? (sprintf q[%s],
+                join q[],
+                map { vec(${$_bitfield{refaddr $self}}, $_, 1) ? q[|] : q[ ] }
+                    0 .. $_torrent{refaddr $self}->piece_count - 1
+                 )
+             : q[NA]
             )
         );
         return defined wantarray ? $dump : print STDERR qq[$dump\n];
@@ -1419,6 +1464,26 @@ ADVANCED
         for (@CONTENTS) { delete $_->{refaddr $self}; }
         return delete $REGISTRY{refaddr $self};
     }
+
+    sub _rc4_crypt {
+        my ($_key, $plaintext) = @_;
+        my @key = unpack q[C*], $_key;
+        my ($_j, $i, $j) = (0, 0, 0);
+        my @S = 0 .. 255;    # KSA
+        for my $_i (0 .. 255) {
+            $_j = ($_j + $key[$_i % @key] + $S[$_i]) & 255;
+            @S[$_i, $_j] = @S[$_j, $_i];
+        }
+        my $rc4_output = sub {    # PRGA
+            $i = ($i + 1) & 255;
+            $j = ($j + $S[$i]) & 255;
+            @S[$i, $j] = @S[$j, $i];
+            return $S[($S[$i] + $S[$j]) & 255];
+        };
+        return pack q[C*],
+            map { ord(substr($plaintext, $_, 1)) ^ $rc4_output->() }
+            0 .. length($plaintext) - 1;
+    }
     1;
 }
 
@@ -1454,6 +1519,12 @@ Returns the Peer ID used to identify this peer.
 
 See also: theory.org (http://tinyurl.com/4a9cuv)
 
+=item C<as_string ( [ VERBOSE ] )>
+
+Returns a 'ready to print' dump of the  object's data structure.  If
+called in void context, the structure is printed to C<STDERR>.
+C<VERBOSE> is a boolean value.
+
 =back
 
 =head1 Author
@@ -1480,6 +1551,6 @@ clarification, see http://creativecommons.org/licenses/by-sa/3.0/us/.
 Neither this module nor the L<Author|/Author> is affiliated with
 BitTorrent, Inc.
 
-=for svn $Id: Peer.pm 39 2008-11-26 15:49:02Z sanko@cpan.org $
+=for svn $Id: Peer.pm 56a7b7c 2009-01-27 02:13:14Z sanko@cpan.org $
 
 =cut
