@@ -1,4 +1,4 @@
-#!C:\perl\bin\perl.exe
+#!/usr/bin/perl -w
 package Net::BitTorrent::Torrent::Tracker::UDP;
 {
     use strict;
@@ -8,13 +8,16 @@ package Net::BitTorrent::Torrent::Tracker::UDP;
     use List::Util qw[sum];
     use Socket qw[inet_aton pack_sockaddr_in];
     use lib q[../../../../../lib];
-    use Net::BitTorrent::Util qw[uncompact];
+    use Net::BitTorrent::Util qw[:compact];
     use version qw[qv];
-    our $SVN = q[$Id: UDP.pm 39 2008-11-26 15:49:02Z sanko@cpan.org $];
-    our $UNSTABLE_RELEASE = 0; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new((qw$Rev: 39 $)[1])->numify / 1000), $UNSTABLE_RELEASE);
+    our $VERSION_BASE = 49; our $UNSTABLE_RELEASE = 3; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new(($VERSION_BASE))->numify / 1000), $UNSTABLE_RELEASE);
     my %REGISTRY = ();
-    my @CONTENTS = \my (%_url, %_tier, %_tid, %_cid, %_outstanding_requests,
-                        %_packed_host, %_event);
+    my @CONTENTS = \my (%_url,                  %_tier,
+                        %_tid,                  %_cid,
+                        %_outstanding_requests, %_packed_host,
+                        %_event,                %_peers,
+                        %_complete,             %_incomplete
+    );
 
     sub new {
         my ($class, $args) = @_;
@@ -46,19 +49,25 @@ package Net::BitTorrent::Torrent::Tracker::UDP;
         else { $packed_host = inet_aton($host) }
         $_packed_host{refaddr $self}
             = pack_sockaddr_in($port, inet_aton($host));
-        $_url{refaddr $self}   = $args->{q[URL]};
-        $_event{refaddr $self} = q[];
-        $_tier{refaddr $self}  = $args->{q[Tier]};
-        $_tid{refaddr $self}   = int(rand() * 26**5);
+        $_url{refaddr $self}        = $args->{q[URL]};
+        $_event{refaddr $self}      = q[];
+        $_tier{refaddr $self}       = $args->{q[Tier]};
+        $_peers{refaddr $self}      = q[];
+        $_complete{refaddr $self}   = 0;
+        $_incomplete{refaddr $self} = 0;
+        $_tid{refaddr $self}        = int(rand() * 26**5);
         weaken $_tier{refaddr $self};
         weaken($REGISTRY{refaddr $self} = $self);
         return $self;
     }
 
+    # Accessors | Public
+    sub url { my ($self) = @_; return $_url{refaddr $self}; }
+
     # Accessors | Private
     sub _packed_host { return $_packed_host{refaddr +shift} }
     sub _tier        { return $_tier{refaddr +shift}; }
-    sub _url         { return $_url{refaddr +shift}; }
+    sub _peers       { return $_peers{refaddr +shift}; }
     sub _client      { return $_tier{refaddr +shift}->_client }
 
     # Methods | Private
@@ -92,17 +101,17 @@ package Net::BitTorrent::Torrent::Tracker::UDP;
                 $self->_client->peerid(),
                 ___pack64($_tier{refaddr $self}->_torrent->downloaded()),
                 ___pack64(
-                   $_tier{refaddr $self}
-                       ->_torrent->raw_data->{q[info]}{q[piece length]} * sum(
-                       split(q[],
-                             unpack(
+                     $_tier{refaddr $self}->_torrent->raw_data(1)
+                         ->{q[info]}{q[piece length]} * sum(
+                         split(q[],
+                               unpack(
                                    q[b*],
                                    ($_tier{refaddr $self}->_torrent->_wanted()
                                         || q[]
                                    )
-                             )
-                       )
-                       )
+                               )
+                         )
+                         )
                 ),
                 ___pack64($_tier{refaddr $self}->_torrent->uploaded()),
                 (  $_event{refaddr $self} eq q[completed] ? 1
@@ -110,7 +119,8 @@ package Net::BitTorrent::Torrent::Tracker::UDP;
                  : $_event{refaddr $self} eq q[stopped]   ? 3
                  : 0
                 ),
-                0, $^T, 200, $self->_client->_tcp_port;
+                0, $^T, 200, $self->_client->_tcp_port
+                || 0;
             $_outstanding_requests{refaddr $self}{$tid} = {Timestamp => time,
                                                            Attempt   => 1,
                                                            Packet => $packet,
@@ -203,10 +213,9 @@ package Net::BitTorrent::Torrent::Tracker::UDP;
             if (length($data) >= 20) {
                 my ($min_interval, $leeches, $seeds, $peers)
                     = unpack(q[N N N a*], $packet);
-                $_tier{refaddr $self}
-                    ->_torrent->_append_compact_nodes($peers);
-                $_tier{refaddr $self}->_set_complete($seeds);
-                $_tier{refaddr $self}->_set_incomplete($leeches);
+                $_peers{refaddr $self}      = $peers;
+                $_complete{refaddr $self}   = $seeds;
+                $_incomplete{refaddr $self} = $leeches;
                 $_tier{refaddr $self}->_torrent->_event(
                                             q[tracker_success],
                                             {Tracker => $self,
@@ -267,7 +276,7 @@ package Net::BitTorrent::Torrent::Tracker::UDP;
         return $_tid{refaddr $self};
     }
 
-    sub _as_string {
+    sub as_string {
         my ($self, $advanced) = @_;
         my $dump = !$advanced ? $$self : sprintf <<'END',
 Net::BitTorrent::Torrent::Tracker::UDP
@@ -338,6 +347,22 @@ constructor should not be used directly.
 
 =back
 
+=head1 Methods
+
+=over
+
+=item C<url ( )>
+
+Returns the related UDP 'URL' according to the original metadata.
+
+=item C<as_string ( [ VERBOSE ] )>
+
+Returns a 'ready to print' dump of the  object's data structure.  If
+called in void context, the structure is printed to C<STDERR>.
+C<VERBOSE> is a boolean value.
+
+=back
+
 =head1 BUGS/TODO
 
 =over 4
@@ -388,6 +413,6 @@ clarification, see http://creativecommons.org/licenses/by-sa/3.0/us/.
 Neither this module nor the L<Author|/Author> is affiliated with
 BitTorrent, Inc.
 
-=for svn $Id: UDP.pm 39 2008-11-26 15:49:02Z sanko@cpan.org $
+=for svn $Id: UDP.pm 56a7b7c 2009-01-27 02:13:14Z sanko@cpan.org $
 
 =cut
