@@ -1,11 +1,10 @@
-#!C:\perl\bin\perl.exe
+#!/usr/bin/perl -w
 package Net::BitTorrent::Version;
 {
     use strict;
     use warnings;
     use version qw[qv];
-    our $SVN = q[$Id: Version.pm 39 2008-11-26 15:49:02Z sanko@cpan.org $];
-    our $VERSION_BASE = 39; our $UNSTABLE_RELEASE = 2; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new(($VERSION_BASE))->numify / 1000), $UNSTABLE_RELEASE);
+    our $VERSION_BASE = 49; our $UNSTABLE_RELEASE = 3; our $VERSION = sprintf(($UNSTABLE_RELEASE ? q[%.3f_%03d] : q[%.3f]), (version->new(($VERSION_BASE))->numify / 1000), $UNSTABLE_RELEASE);
     our $PRODUCT_TOKEN = qq[Net::BitTorrent $VERSION];
 
     sub gen_peerid {
@@ -21,20 +20,14 @@ package Net::BitTorrent::Version;
                       ->[rand(66)]
                       } 1 .. 8
                  ),
-                 q[Azumi]
+                 q[Riana]
              )
             )
         );
     }
 
     sub gen_node_id {
-        return
-            pack(q[a20],
-                 (join(q[],
-                       map { [q[a] .. q[z], q[A] .. q[Z]]->[rand(52)] }
-                           1 .. 20)
-                 )
-            );
+        return pack(q[H40], join(q[], map { rand(12) } 1 .. 40));
     }
     1;
 }
@@ -80,22 +73,18 @@ L<header|/Header> which may be used to identify the software and its
 version, and the L<signature|/Signature> which is... well, it's junk.
 Consider this example:
 
- NB004S-rogzGB1v--SVN
+ NB004S-rogzGB1v--git
 
-Here, C<NB004S> is the header and C<-rogzGB1v--SVN> is the trailing
+Here, C<NB004S> is the header and C<-rogzGB1v--git> is the trailing
 signature.
 
 =head3 Header
 
 The header consists of two uppercase characters ('C<NB>') followed by
-three digits (with leading zeros) representing the SVN revision, and a
-single character used to (potentially) indicate stability.
-
-If the client is a stable build, the sixth character is the capital
-letter 'C<C>' (as these are usually obtained from CPAN).  If the client
-is a version checked out from SVN, the sixth character is the capital
-letter 'C<S>'.  Any other characters in the sixth byte are unsupported
-and may indicate a bad client.
+three digits (with leading zeros) representing the distribution version
+and a single character used to (potentially) indicate stability:
+C<S>table or C<U>nstable.  Any deviation from this indicates a bad
+client.  Recent stable builds will be found on CPAN.
 
 =head3 Signature
 
@@ -107,46 +96,41 @@ in the following range:
 That is, all uppercase and lowercase letters, decimal digits, as well as
 the hyphen, period, underscore, and tilde (66 total).  These are all
 characters allowed in a URL without being encoded (referred to as
-"Unreserved Characters" in rfc://3986) to reduce weight on the tracker
-during announce.
+"Unreserved Characters" in L<rfc 3986|/"rfc3986">).
 
 =head2 Version Numbers and Stability
 
-Version numbers will be some value less than one (C<1>) with the revision
-number in the three significant decimal places.
+Distribution version numbers will be some value less than one (C<1>) with
+the revision number in the three significant decimal places.
 
-Stable revisions will have the sixth char set to 'C<S>' (capital
-letter 's'). The most recent of these stable builds will be found on
-CPAN.
-
-Unstable revisions will have the sixth char set to 'C<U>' (capital
-letter 'u').  These will most likely be SVN checkouts and temporary
-uploads to CPAN where the package's filename matches  C<m[\d\.\d+_\d]>.
-See the PAUSE FAQ section entitled "Developer Releases"
-(L<http://www.cpan.org/modules/04pause.html>).
+Recent stable builds will be found on CPAN.  Unstable builds will most
+likely be private builds or temporary uploads to CPAN where the package
+name matches C<m[\d\.\d+_\d]>.  See the PAUSE FAQ section entitled
+"Developer Releases" (L<http://www.cpan.org/modules/04pause.html>).
 
 =head2 Examples
 
 =over
 
-=item C<NB393S-04.9EraQ--SVN>
+=item C<NB393S-04.9EraQ--git>
 
-This would be the stable CPAN release C<v0.393>/SVN r393.  The C<--SVN>
+This would be the stable CPAN-ready release C<v0.393>.  The C<--git>
 signature does not imply an unstable build.
 
 =item C<NB003X-9E-ayR6-IE<lt>3BT>
 
-Improper Peer ID; the sixth char is neither 'C<S>' nor 'C<U>'.
+Improper Peer ID; the sixth char is neither 'C<S>' nor 'C<U>' and E<lt>
+is a reserved character according to L<rfc: 3986|/"rfc3986">.
 
-=item C<NB065C--09Egae69sy8W>
+=item C<NB065U--09Egae69sy8W>
 
-Completely legal Peer ID generated by unstable SVN/Dev r65.
+Completely legal Peer ID generated by unstable build around C<v65>.
 
 =back
 
 =head1 See Also
 
-RFC 3986 (URI: Generic Syntax)
+X<rfc3986>RFC 3986 (URI: Generic Syntax)
 Section 2.3. "Unreserved Characters"
 (http://tools.ietf.org/html/rfc3986#section-2.3)
 
@@ -160,7 +144,7 @@ http://slashdot.org/comments.pl?sid=997033&cid=25390887
 This document and the specification behind it are subject to change.
 All modifications will be documented in the Changes file included with
 this distribution.  All versions of this file can be found in the
-project's SVN repository.
+project's git repository.
 
 =head1 Author
 
@@ -186,6 +170,6 @@ clarification, see http://creativecommons.org/licenses/by-sa/3.0/us/.
 Neither this module nor the L<Author|/Author> is affiliated with
 BitTorrent, Inc.
 
-=for svn $Id: Version.pm 39 2008-11-26 15:49:02Z sanko@cpan.org $
+=for svn $Id: Version.pm 56a7b7c 2009-01-27 02:13:14Z sanko@cpan.org $
 
 =cut
